@@ -4,29 +4,26 @@
 using namespace Rcpp;
 using namespace arma;
 
-//' Rho Koenker (internal function)
- //'
- //' @param x Numeric vector
- //' @param tau Numeric scalar, percentile (between 0 and 1)
- //'
- //' @return A numeric vector with the Koenker rho function applied element-wise.
- //' @keywords internal
- //' @noRd
- // [[Rcpp::export]]
+
+//' @name Rho Koenker 
+//'
+//' @param x Numeric vector
+//' @param tau Numeric scalar, percentile (between 0 and 1)
+//'
+//' @return A numeric vector with the Koenker rho function applied element-wise.
+//' @keywords internal
+//' @noRd
+// [[Rcpp::export]]
 arma::vec rho_koenker(arma::vec x, double tau){
-  int n = x.n_elem;
-  arma::vec y(n);
-  for(int i = 0; i < n; ++i){
-    if(x(i)<0){
-      y(i) = x(i)*(tau-1);
-    } else {
-      y(i) = x(i)*tau;
-    }
-  }
-  return(y);
+  arma::vec y = x;                             // copy x
+  arma::uvec neg = arma::find(x < 0);          // indices where x < 0
+  arma::uvec pos = arma::find(x >= 0);         // indices where x >= 0
+  y(neg) *= (tau - 1.0);
+  y(pos) *= tau;
+  return y;
 }
 
-//' Loss quantile regression
+//' @name Loss quantile regression
 //'
 //' @param beta initial values
 //' @param x design matrix
@@ -49,7 +46,7 @@ double loss_qr(arma::vec beta, arma::mat x, arma::vec y, double tau, int N, int
   return(eta);
 }
 
-//' Loss quantile regression with fixed effects
+//' @name Loss quantile regression with fixed effects
 //'
 //' @param theta initial values
 //' @param x design matrix
@@ -71,15 +68,17 @@ double loss_qrfe(arma::vec theta,arma::mat x,arma::vec y,arma::mat z,double tau,
   arma::vec res(n);
   arma::vec rho(n);
   beta = theta.subvec(0, d - 1);
-  alpha.head(mm - 1) = theta.subvec(d, d + mm - 2);     // alpha: next mm - 1 values
-  alpha(mm - 1) = -arma::accu(alpha.head(mm - 1));      // sum-to-zero constraint
+  alpha = theta.subvec(d, d + mm - 1);     
+  if (arma::all(x.col(0) == 1.0)) {
+    alpha -= arma::mean(alpha);  // subtract mean to enforce sum(alpha) = 0
+  }
   res = y -  (z * alpha) -  (x * beta);
   rho = rho_koenker(res,tau);
   eta = accu(rho);
   return(eta);
 }
 
-//' Loss lasso quantile regression with fixed effects
+//' @name Loss lasso quantile regression with fixed effects
 //'
 //' @param theta initial values
 //' @param x design matrix
@@ -102,8 +101,10 @@ double loss_lqr(arma::vec theta,arma::mat x,arma::vec y,arma::mat z,double tau,i
   arma::vec res(n);
   arma::vec rho(n);
   beta = theta.subvec(0, d - 1);
-  alpha.head(mm - 1) = theta.subvec(d, d + mm - 2);     // alpha: next mm - 1 values
-  alpha(mm - 1) = -arma::accu(alpha.head(mm - 1));      // sum-to-zero constraint
+  alpha = theta.subvec(d, d + mm - 1); 
+  if (arma::all(x.col(0) == 1.0)) {
+    alpha -= arma::mean(alpha);  // subtract mean to enforce sum(alpha) = 0
+  }
   res = y -  (z * alpha) -  (x * beta);
   rho = rho_koenker(res,tau);
   eta = accu(rho) + lambda * accu(abs(alpha));
@@ -138,16 +139,15 @@ double loss_alqr(arma::vec theta,arma::mat x,arma::vec y,arma::mat z,double tau,
   arma::vec res(n);
   arma::vec rho(n);
   arma::vec theta2(d + mm);
-
   beta = theta.subvec(0, d - 1);                  // First d are beta
-  alpha.head(mm - 1) = theta.subvec(d, d + mm - 2);         // Fill first mm-1 alpha
-  alpha(mm - 1) = -arma::accu(alpha.head(mm - 1));          // Sum-to-zero constraint
+  alpha = theta.subvec(d, d + mm - 1);
+  if (arma::all(x.col(0) == 1.0)) {
+    alpha -= arma::mean(alpha);  // subtract mean to enforce sum(alpha) = 0
+  }
   theta2.head(d) = beta;
   theta2.tail(mm) = alpha;
-
   res = y - z * alpha - x * beta;
   rho = rho_koenker(res, tau);
-
   penalty = arma::accu(arma::abs(theta2 / w));       // element-wise division
   eta = arma::accu(rho) + lambda * penalty;
   return eta;
